package jp.allabout.android;

import java.io.IOException;
import java.util.List;
import java.util.Locale;

import jp.testargmap.CommonLib;
import jp.testargmap.R;
import jp.testargmap.TargetInformation;
import jp.testargmap.TestArgMapActivity.IconOverlay;

import com.google.android.maps.GeoPoint;
import com.google.android.maps.MapController;
import com.google.android.maps.MapView;
import com.google.android.maps.Overlay;
import com.google.android.maps.Projection;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Point;
import android.location.Address;
import android.location.Geocoder;
import android.location.Location;
import android.location.LocationManager;
import android.os.Bundle;
import android.view.Gravity;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.ZoomControls;

public class ArgMap extends Activity {

	//地図の初期値
	static final int INITIAL_ZOOM_LEVEL = 15;

	//位置情報の通知範囲
	static final long MIN_TIME = 0;
	static final float MIN_DISTANCE = 0;

	//位置情報を取得するためのマネージャ
	LocationManager mLocationManager;

	//位置変化時に地図を移動させるコントローラ
	MapController mMapController;

	//検索用のクラス
	Geocoder mGeoCoder;

	//地名の表示用テキストビュー
	TextView mStatusText;

	//目的地の位置情報
	double targetLatitude;
	double targetLongitude;
	boolean thereIsTarget = false;

	//現在の位置情報
	double myLatitude;
	double myLongitude;

	//ボタン
	Button button0;
	Button button1;

    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);

        //MapViewからコントローラを取り出す
        MapView map_view = (MapView) findViewById(R.id.mapview);

        //地図のズームを制御するためのズームコントローラを設置
        ZoomControls zoom = (ZoomControls) map_view.getZoomControls();
        ViewGroup.LayoutParams layout =
        	new ViewGroup.LayoutParams(
        			ViewGroup.LayoutParams.FILL_PARENT,ViewGroup.LayoutParams.FILL_PARENT);
        zoom.setLayoutParams(layout);
        zoom.setGravity(Gravity.BOTTOM | Gravity.CENTER_HORIZONTAL);
        map_view.addView(zoom);

        //位置とズームレベルの初期状態を設定する
        mMapController = map_view.getController();
        mMapController.setZoom(INITIAL_ZOOM_LEVEL);

        //検索用クラス
        mGeoCoder =
        	new Geocoder(getApplicationContext(),Locale.JAPAN);
        //テキスト表示
        mStatusText = (TextView)findViewById(R.id.TextView01);

        //位置情報取得用のクラス
        mLocationManager = (LocationManager)getSystemService(Context.LOCATION_SERVICE);
        Location loc = mLocationManager.getLastKnownLocation(LocationManager.GPS_PROVIDER);
        myLatitude = loc.getLatitude();
        myLongitude = loc.getLongitude();
		GeoPoint gp = new GeoPoint( (int)(myLatitude*1E6),
									(int)(myLongitude*1E6));
        mMapController.setCenter(gp);
        //ボタンを取り出して、リスナー登録する
        button0 = (Button)findViewById(R.id.Button01);
        button0.setOnClickListener(this);

        button1 = (Button)findViewById(R.id.Button02);
        button1.setOnClickListener(this);
    }

    public void onResume(){
    	super.onResume();
    	//最後に取得した位置があれば表示する
    	Location loc = mLocationManager.getLastKnownLocation(LocationManager.GPS_PROVIDER);
    	if(loc != null){
    		onLocationChanged(loc);
    	}
    	//現在位置が変化したらメソッドが呼び出されるよう登録する
    	mLocationManager.requestLocationUpdates(LocationManager.GPS_PROVIDER, MIN_TIME,MIN_DISTANCE, this);
    }

    protected void onPause(){
    	super.onPause();
    	//位置情報の通知の登録を解除する
    	mLocationManager.removeUpdates(this);
    }

	public void onLocationChanged(Location location) {
		// TODO 自動生成されたメソッド・スタブ
		//LocationをGeoPointに変更して地図の表示を変更する
		myLatitude = location.getLatitude();
		myLongitude = location.getLongitude();
		//mMapController.animateTo(gp);

	}

	public void onProviderDisabled(String provider) {
		// TODO 自動生成されたメソッド・スタブ

	}

	public void onProviderEnabled(String provider) {
		// TODO 自動生成されたメソッド・スタブ

	}

	public void onStatusChanged(String provider, int status, Bundle extras) {
		// TODO 自動生成されたメソッド・スタブ

	}

	@Override
	protected boolean isRouteDisplayed() {
		// TODO 自動生成されたメソッド・スタブ
		return false;
	}


	public void onClick(View view) {
		// TODO 自動生成されたメソッド・スタブ
		if(view == button0){
		EditText edit = (EditText)findViewById(R.id.EditText01);
		StringBuffer buff = new StringBuffer();
		try {
			List<Address> addrs0 = mGeoCoder.getFromLocationName(
					edit.getText().toString(), 1);
			if(addrs0.size() == 0){
				//アラートダイアログ
				AlertDialog.Builder dBuilder = new AlertDialog.Builder(this);
				dBuilder.setMessage("条件に一致する住所は見つかりませんでした。");

				dBuilder.setIcon(R.drawable.icon);

				dBuilder.setPositiveButton("OK",new DialogInterface.OnClickListener(){
					public void onClick(DialogInterface dialog, int which){
						dialog.dismiss();
					}
				});
				dBuilder.show();
			}

			for(Address addr0 : addrs0){
				targetLatitude = addr0.getLatitude();
				targetLongitude = addr0.getLongitude();
				TargetInformation.INSTANCE.latitude = targetLatitude;
				TargetInformation.INSTANCE.longitude = targetLongitude;
				List<Address> addrs1 = mGeoCoder.getFromLocation(targetLatitude, targetLongitude, 1);
				buff.append("目的地:\n");
				int index0 = addr0.getMaxAddressLineIndex();
				StringBuffer buff0 = new StringBuffer();
				for(int i0=0;i0 <=index0; i0++){
					buff0.append(addr0.getAddressLine(i0));
					buff0.append(" ");
				}
				buff.append(buff0.toString());
				TargetInformation.INSTANCE.name = buff.toString();
				buff.delete(0, buff.length()-1);
				for(Address addr1 : addrs1){
					GeoPoint gp = new GeoPoint( (int)(targetLatitude*1E6),
							(int)(targetLongitude*1E6));
					mMapController.animateTo(gp);
					setOverlay(gp);
					//buff.append("目的地:\n");
					buff.append(/*addr1.getFeatureName()*//*edit.getText().toString() + */"\n住所:\n");
					int index = addr1.getMaxAddressLineIndex();
					for(int i = 0; i <= index; ++i){
						buff0.append(addr1.getAddressLine(i));
						buff0.append(" ");
					}
					buff.append(buff0.toString());
					TargetInformation.INSTANCE.address = buff0.toString();
					buff.append("\n");
					double distance = CommonLib.getDistance(
							myLatitude, myLongitude, targetLatitude, targetLongitude);
					buff.append("距離:\n" + distance + "\n");
				}
			}
			System.out.println("aaa" + edit.getText().toString() + "\n" + targetLatitude + ", " + targetLongitude);
		} catch (IOException e) {
			// TODO 自動生成された catch ブロック
			e.printStackTrace();
		}
		mStatusText.setText(buff.toString());
		}
		else if(view == button1){

		}
	}


    private void setOverlay(GeoPoint point) {

        // Overlayクラスを生成する
        Bitmap icon = BitmapFactory.decodeResource(getResources(),
                                                   R.drawable.icon);
        IconOverlay overlay = new IconOverlay(icon, point);

        // 生成したOverlayクラスを追加する
        MapView map_view = (MapView) findViewById(R.id.mapview);
        List<Overlay> overlays = map_view.getOverlays();
        overlays.clear();
        overlays.add(overlay);
    }

    // 地図上に表示されるオーバーレイ
    private class IconOverlay extends Overlay {

        // 描画するアイコン
        Bitmap mIcon;
        int mOffsetX;
        int mOffsetY;

        // アイコンを表示する位置
        GeoPoint mPoint;

       IconOverlay(Bitmap icon, GeoPoint initial) {
            // アイコンと、アイコンの中心のオフセット
            mIcon = icon;
            mOffsetX = 0 - icon.getWidth() / 2;
            mOffsetY = 0 - icon.getHeight() / 2;
            mPoint = initial;
        }



        @Override
        public void draw(Canvas canvas, MapView mapView,
                         boolean shadow) {
            super.draw(canvas, mapView, shadow);
            if (!shadow) {
                // 地図上の場所と、描画用のCanvasの座標の変換
                Projection projection = mapView.getProjection();
                Point point = new Point();
                projection.toPixels(mPoint, point);
                point.offset(mOffsetX, mOffsetY);
                // アイコンを描画
                canvas.drawBitmap(mIcon, point.x, point.y, null);
            }
        }
    }
}